//	LN^LCD I2CA_v^F}X^[Cu
//	wt]xH쎺	http://jsdiy.web.fc2.com

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "I2cCharLcdLib.h"

static uint8_t SlaveAddress = 0xFF;

#define HiByte(w)	(((w) >> 8) & 0xFF)
#define LoByte(w)	((w) & 0xFF)
#define BoolValue(a)	((a) ? 1 : 0)

//Rg[oCg1̃f[^oCg𑗐M
static	uint8_t	SendByteP1(uint8_t ctrlByte, uint8_t dataByte)
{
	uint8_t	isOK =
		TWI_Start() && TWI_SlaveAddrW(SlaveAddress) &&
		TWI_WriteByte(ctrlByte) && TWI_WriteByte(dataByte);
	TWI_Stop();
	return BoolValue(isOK);
}

//Rg[oCg2̃f[^oCg𑗐M
static	uint8_t	SendByteP2(uint8_t ctrlByte, uint8_t dataByte1, uint8_t dataByte2)
{
	uint8_t	isOK =
		TWI_Start() && TWI_SlaveAddrW(SlaveAddress) &&
		TWI_WriteByte(ctrlByte) && TWI_WriteByte(dataByte1) && TWI_WriteByte(dataByte2);
	TWI_Stop();
	return BoolValue(isOK);
}

//Rg[oCgƃf[^oCg̔z𑗐M
static	uint8_t	SendByteBlock(uint8_t ctrlByte, const uint8_t* dataBytes, uint8_t length)
{
	uint8_t	i;
	uint8_t	isOK =
		TWI_Start() && TWI_SlaveAddrW(SlaveAddress) &&
		TWI_WriteByte(ctrlByte);
		
	for (i = 0; isOK && i < length; i++)
		isOK = TWI_WriteByte(dataBytes[i]);
	
	TWI_Stop();
	return BoolValue(isOK);
}

//ΏۂLCDfoCX
//E{foCX̓X[uAhXςI2CoXɕڑ邱ƂłB
//	ꂼ̃foCXɑ΂Ă̊֐ĂяoKvB
//EI2CoXɖ{foCX1ڑĂȂꍇÅ֐1ĂяoΏ\B
uint8_t	I2CCHARLCD_Init(uint8_t slaveAddress, uint8_t columnSize, uint8_t rowSize)
{
	I2CCHARLCD_SlaveSelect(slaveAddress);
	return SendByteP2(LCD_FUNC_INIT, columnSize, rowSize);
}

//ΏۂLCDfoCX؂ւ
//E{foCX̓X[uAhXςI2CoXɕڑ邱ƂłB
//	̏ꍇɁAǂLCDfoCX𑀍삷̂A̓sx̊֐Ő؂ւB
//EI2CoXɖ{foCX1ڑĂȂꍇÅ֐Ăяoʂ͂ȂB
void	I2CCHARLCD_SlaveSelect(uint8_t slaveAddress)
{
	SlaveAddress = slaveAddress;
}

//HD44780x̃R}h𑗐MiڍׂHD44780̃f[^V[g̃R}hꗗQƁj
uint8_t	I2CCHARLCD_SendCmd(uint8_t cmd)
{
	return SendByteP1(RS_CMD, cmd);
}

//HD44780x̃f[^𑗐MiڍׂHD44780̃f[^V[g̃R}hꗗQƁj
uint8_t	I2CCHARLCD_SendData(uint8_t data)
{
	return SendByteP1(RS_DATA, data);
}

//s\̓Zbg
uint8_t	I2CCHARLCD_SetLineMode(uint8_t lineMode, uint8_t cursorFlag)
{
	return SendByteP2(LCD_FUNC_SETLINEMODE, lineMode, cursorFlag);
}

//̕\ʒuZbg
uint8_t	I2CCHARLCD_SetPos(uint8_t row_y, uint8_t column_x)
{
	return SendByteP2(LCD_FUNC_SETPOS_XY, column_x, row_y);
}

//LCDʂ
uint8_t	I2CCHARLCD_Cls()
{
	return I2CCHARLCD_SendCmd(LCD_CMD_CLEAR_DISPLAY);
}

//1\
uint8_t	I2CCHARLCD_PutChar(char cc)
{
	return SendByteP1(LCD_FUNC_PUTCHAR, cc);
}

//\
uint8_t	I2CCHARLCD_PutString(const char* str)
{
	uint8_t length;
	for (length = 0; length < MAX_DATA_LENGTH && str[length] != '\0'; length++);
	return SendByteBlock(LCD_FUNC_PUTCHAR, (uint8_t*)str, length);
}

//vÖɊi[ꂽ\
uint8_t	I2CCHARLCD_PutStringP(const char* strpgm)
{
	uint8_t i, cc;
	uint8_t	isOK =
		TWI_Start() && TWI_SlaveAddrW(SlaveAddress) &&
		TWI_WriteByte(LCD_FUNC_PUTCHAR);
	
	for (i = 0; isOK && i < MAX_DATA_LENGTH; i++)
	{
		cc = pgm_read_byte(strpgm++);
		if (cc == '\0') break;
		isOK = TWI_WriteByte(cc);
	}
	
	TWI_Stop();
	return BoolValue(isOK);
}

//萮\i10ij	8bit/16bitp
uint8_t	I2CCHARLCD_PutInt(int16_t n)
{
	return SendByteP2(LCD_FUNC_PUTINT, HiByte(n), LoByte(n));
}

//Ȃ\i10ij	8bit/16bitp
uint8_t	I2CCHARLCD_PutUInt(uint16_t n)
{
	return SendByteP2(LCD_FUNC_PUTUINT, HiByte(n), LoByte(n));
}

//0l216i\	ړ'0x'ڔ'h'͕tȂ
uint8_t	I2CCHARLCD_PutByte(uint8_t n)
{
	return SendByteP1(LCD_FUNC_PUTBYTE, n);
}

//0l416i\	ړ'0x'ڔ'h'͕tȂ
uint8_t	I2CCHARLCD_PutWord(uint16_t n)
{
	return SendByteP2(LCD_FUNC_PUTBYTE, HiByte(n), LoByte(n));
}

//[U[`iڍׂHD44780̃f[^V[g̃R}hꗗQƁj
//id:	0-7iLN^R[hj
//dots:	uint8_t[8]i5x8̃hbgp^[j	z̑Ó`FbN͏ȗ
uint8_t	I2CCHARLCD_DefChar(uint8_t id, const uint8_t* dots)
{
	uint8_t	i, isOK = 1;
	for (i = 0; isOK && i < 8; i++)
	{
		isOK = I2CCHARLCD_SendCmd(LCD_CMD_SET_CGRAM_ADRS | (id << 3) | i)
			&& I2CCHARLCD_SendData(dots[i]);
	}
	return BoolValue(isOK);
}
